=== modified file 'cronscripts/publishing/cron.germinate'
--- cronscripts/publishing/cron.germinate	2010-09-06 09:46:01 +0000
+++ cronscripts/publishing/cron.germinate	2010-10-20 13:07:20 +0000
@@ -6,12 +6,12 @@
 set -e
 set -u
 
-ARCHIVEROOT=/srv/launchpad.net/ubuntu-archive/ubuntu
+ARCHIVEROOT=${TEST_ARCHIVEROOT:-/srv/launchpad.net/ubuntu-archive/ubuntu}
 MISCROOT=$ARCHIVEROOT/../ubuntu-misc
 LOCKROOT=$ARCHIVEROOT/..
 GERMINATEROOT=$ARCHIVEROOT/../ubuntu-germinate
 
-LAUNCHPADROOT=/srv/launchpad.net/codelines/current
+LAUNCHPADROOT=${TEST_LAUNCHPADROOT:-/srv/launchpad.net/codelines/current}
 MAINTAINCE_CHECK=$LAUNCHPADROOT/cronscripts/publishing/maintenance-check.py
 
 ## Check to see if another germinate run is in progress
@@ -28,7 +28,6 @@
 
 trap cleanup EXIT
 
-LAUNCHPADROOT=/srv/launchpad.net/codelines/current
 suite=`$LAUNCHPADROOT/scripts/ftpmaster-tools/lp-query-distro.py development`
 
 echo -n "Running germinate... "
@@ -126,10 +125,27 @@
 done
 echo " done."
 
-# now generate the Supported extra overrides
-$MAINTAINCE_CHECK $suite > "$MISCROOT/more-extra.override.$suite.main.supported" 2> _maintenance-check.stderr
-if [ $? -eq 0 ]; then
-    cat "$MISCROOT/more-extra.override.$suite.main.supported" >> "$MISCROOT/more-extra.override.$suite.main.new"
-fi
-
 mv -f "$MISCROOT/more-extra.override.$suite.main.new" "$MISCROOT/more-extra.override.$suite.main"
+
+# Now generate the Supported extra overrides for all supported distros.
+SUITES=`$LAUNCHPADROOT/scripts/ftpmaster-tools/lp-query-distro.py supported`
+for supported_suite in $SUITES; do
+    echo -n "Running maintenance-check for $supported_suite... "
+    # The support timeframe information is stored here
+    SUPPORTED="$MISCROOT/more-extra.override.$supported_suite.main.supported"
+    # This is the target override file that contains germinate plus support info
+    TARGET="$MISCROOT/more-extra.override.$supported_suite.main"
+    # Debug/Log information 
+    LOG="_maintenance-check.$supported_suite.stderr"
+    if $MAINTAINCE_CHECK $supported_suite > $SUPPORTED 2> $LOG; then
+        # The target file may be missing on the server and the script should
+        # not fail if that is the case.
+        touch $TARGET
+        # Remove old "Supported" info from extra-overrides as it may be 
+        # stale now and we replace it with fresh content below.
+        sed /"^.* Supported"/d $TARGET > ${TARGET}.new
+        cat $SUPPORTED >> ${TARGET}.new
+        mv ${TARGET}.new $TARGET
+    fi
+    echo " done"
+done

=== modified file 'cronscripts/publishing/maintenance-check.py'
--- cronscripts/publishing/maintenance-check.py	2010-05-28 16:17:51 +0000
+++ cronscripts/publishing/maintenance-check.py	2010-11-04 14:48:20 +0000
@@ -1,7 +1,7 @@
 #!/usr/bin/python
 #
 # python port of the nice maintainace-check script by  Nick Barcet
-# 
+#
 # taken from:
 #  https://code.edge.launchpad.net/~mvo/ubuntu-maintenance-check/python-port
 # (where it will vanish once taken here)
@@ -9,7 +9,7 @@
 # this warning filter is only needed on older versions of python-apt,
 # once the machine runs lucid it can be removed
 import warnings
-warnings.filterwarnings("ignore","apt API not stable yet")
+warnings.filterwarnings("ignore", "apt API not stable yet")
 import apt
 warnings.resetwarnings()
 
@@ -29,21 +29,21 @@
 #  - distros "kubuntu", "edubuntu" and "netbook" need to be
 #    considered *but* only follow SUPPORT_TIMEFRAME
 #  - anything that is in armel follows SUPPORT_TIMEFRAME
-#  
+#
 
 # codename of the lts releases
-LTS_RELEASES = [ "dapper", "hardy", "lucid" ]
+LTS_RELEASES = ["dapper", "hardy", "lucid"]
 
 # architectures that are full supported (including LTS time)
-PRIMARY_ARCHES =  ["i386", "amd64"]
+PRIMARY_ARCHES = ["i386", "amd64"]
 
 # architectures we support (but not for LTS time)
 SUPPORTED_ARCHES = PRIMARY_ARCHES + ["armel"]
 
 # what defines the seeds is documented in wiki.ubuntu.com/SeedManagement
-SERVER_SEEDS = [ "supported-server", "server-ship"]
+SERVER_SEEDS = ["supported-server", "server-ship"]
 DESKTOP_SEEDS = ["ship", "supported-desktop", "supported-desktop-extra"]
-SUPPORTED_SEEDS = [ "all" ]
+SUPPORTED_SEEDS = ["all"]
 
 # normal support timeframe
 # time, seeds, arches
@@ -60,10 +60,25 @@
 ]
 
 # distro names and if they get LTS support (order is important)
-DISTRO_NAMES_AND_LTS_SUPPORT = [ ("ubuntu",   True),
-                                 ("kubuntu",  True),
-                                 ("netbook",  False),
-                               ]
+DISTRO_NAMES_AND_LTS_SUPPORT = [
+    ("ubuntu", True),
+    ("kubuntu", True),
+    ("netbook", False),
+    ]
+
+# Names of the distribution releases that are not supported by this
+# tool. All later versions are supported.
+UNSUPPORTED_DISTRO_RELEASED = [
+    "dapper",
+    "edgy",
+    "feisty",
+    "gutsy",
+    "hardy",
+    "intrepid",
+    "jaunty",
+    "karmic",
+    ]
+
 
 # germinate output base directory
 BASE_URL = "http://people.canonical.com/~ubuntu-archive/germinate-output/"
@@ -79,6 +94,7 @@
 # support timeframe tag used in the Packages file
 SUPPORT_TAG = "Supported"
 
+
 def get_binaries_for_source_pkg(srcname):
     """ Return all binary package names for the given source package name.
 
@@ -92,38 +108,41 @@
             pkgnames.add(binary)
     return pkgnames
 
+
 def expand_src_pkgname(pkgname):
     """ Expand a package name if it is prefixed with src.
 
     If the package name is prefixed with src it will be expanded
     to a list of binary package names. Otherwise the original
     package name will be returned.
-    
+
     :param pkgname: The package name (that may include src:prefix).
-    :return: A list of binary package names (the list may be one element long).
+    :return: A list of binary package names (the list may be one element
+             long).
     """
     if not pkgname.startswith("src:"):
         return [pkgname]
     return get_binaries_for_source_pkg(pkgname.split("src:")[1])
 
+
 def create_and_update_deb_src_source_list(distroseries):
     """ Create sources.list and update cache.
 
-    This creates a sources.list file with deb-src entries for a given 
+    This creates a sources.list file with deb-src entries for a given
     distroseries and apt.Cache.update() to make sure the data is up-to-date.
     :param distro: The code name of the distribution series (e.g. lucid).
     :return: None
     :raises: IOError: When cache update fails.
     """
     # apt root dir
-    rootdir="./aptroot.%s" % distroseries
-    sources_list_dir = os.path.join(rootdir, "etc","apt")
+    rootdir = "./aptroot.%s" % distroseries
+    sources_list_dir = os.path.join(rootdir, "etc", "apt")
     if not os.path.exists(sources_list_dir):
         os.makedirs(sources_list_dir)
-    sources_list = open(os.path.join(sources_list_dir, "sources.list"),"w")
+    sources_list = open(os.path.join(sources_list_dir, "sources.list"), "w")
     for pocket in [
-        "%s" % distroseries, 
-        "%s-updates" % distroseries, 
+        "%s" % distroseries,
+        "%s-updates" % distroseries,
         "%s-security" % distroseries]:
         sources_list.write(
             "deb-src %s %s main restricted\n" % (
@@ -134,13 +153,13 @@
     sources_list.close()
     # create required dirs/files for apt.Cache(rootdir) to work on older
     # versions of python-apt. once lucid is used it can be removed
-    for d in  ["var/lib/dpkg", 
-               "var/cache/apt/archives/partial",
-               "var/lib/apt/lists/partial"]:
-        if not os.path.exists(os.path.join(rootdir,d)):
-            os.makedirs(os.path.join(rootdir,d))
-    if not os.path.exists(os.path.join(rootdir,"var/lib/dpkg/status")):
-        open(os.path.join(rootdir,"var/lib/dpkg/status"),"w")
+    for d in ["var/lib/dpkg",
+              "var/cache/apt/archives/partial",
+              "var/lib/apt/lists/partial"]:
+        if not os.path.exists(os.path.join(rootdir, d)):
+            os.makedirs(os.path.join(rootdir, d))
+    if not os.path.exists(os.path.join(rootdir, "var/lib/dpkg/status")):
+        open(os.path.join(rootdir, "var/lib/dpkg/status"), "w")
     # open cache with our just prepared rootdir
     cache = apt.Cache(rootdir=rootdir)
     try:
@@ -148,24 +167,28 @@
     except SystemError:
         logging.exception("cache.update() failed")
 
+
 def get_structure(distroname, version):
     """ Get structure file conent for named distro and distro version.
-    
+
     :param name: Name of the distribution (e.g. kubuntu, ubuntu, xubuntu).
     :param version: Code name of the distribution version (e.g. lucid).
     :return: List of strings with the structure file content
     """
-    f = urllib2.urlopen("%s/%s.%s/structure" % (BASE_URL, distroname, version))
+    f = urllib2.urlopen("%s/%s.%s/structure" % (
+            BASE_URL, distroname, version))
     structure = f.readlines()
     f.close()
     return structure
 
+
 def expand_seeds(structure, seedname):
     """ Expand seed by its dependencies using the strucure file.
 
     :param structure: The content of the STRUCTURE file as string list.
     :param seedname: The name of the seed as string that needs to be expanded.
-    :return: a set() for the seed dependencies (excluding the original seedname)
+    :return: a set() for the seed dependencies (excluding the original
+             seedname)
     """
     seeds = []
     for line in structure:
@@ -175,9 +198,10 @@
                 seeds += expand_seeds(structure, seed)
     return set(seeds)
 
+
 def get_packages_for_seeds(name, distro, seeds):
     """
-    get packages for the given name (e.g. ubuntu) and distro release 
+    get packages for the given name (e.g. ubuntu) and distro release
     (e.g. lucid) that are in the given list of seeds
     returns a set() of package names
     """
@@ -185,7 +209,7 @@
     for bseed in seeds:
         for seed in [bseed]: #, bseed+".build-depends", bseed+".seed"]:
             pkgs_in_seeds[seed] = set()
-            seedurl = "%s/%s.%s/%s" % (BASE_URL,name, distro, seed)
+            seedurl = "%s/%s.%s/%s" % (BASE_URL, name, distro, seed)
             logging.debug("looking for '%s'" % seedurl)
             try:
                 f = urllib2.urlopen(seedurl)
@@ -204,6 +228,7 @@
                 logging.error("seed %s failed (%s)" % (seedurl, e))
     return pkgs_in_seeds
 
+
 def what_seeds(pkgname, seeds):
     in_seeds = set()
     for s in seeds:
@@ -211,6 +236,7 @@
             in_seeds.add(s)
     return in_seeds
 
+
 def compare_support_level(x, y):
     """
     compare two support level strings of the form 18m, 3y etc
@@ -218,9 +244,10 @@
     :parm y: the second support level
     :return: negative if x < y, zero if x==y, positive if x > y
     """
+
     def support_to_int(support_time):
         """
-        helper that takes a support time string and converts it to 
+        helper that takes a support time string and converts it to
         a integer for cmp()
         """
         # allow strings like "5y (kubuntu-common)
@@ -233,7 +260,9 @@
             raise ValueError("support time '%s' has to end with y or m" % x)
     return cmp(support_to_int(x), support_to_int(y))
 
-def get_packages_support_time(structure, name, pkg_support_time, support_timeframe_list):
+
+def get_packages_support_time(structure, name, pkg_support_time,
+                              support_timeframe_list):
     """
     input a structure file and a list of pair<timeframe, seedlist>
     return a dict of pkgnames -> support timeframe string
@@ -255,18 +284,21 @@
                                 pkg, old_timeframe, timeframe))
                         pkg_support_time[pkg] = timeframe
                 if options.with_seeds:
-                    pkg_support_time[pkg] += " (%s)" % ", ".join(what_seeds(pkg, pkgs_in_seeds))
+                    pkg_support_time[pkg] += " (%s)" % ", ".join(
+                        what_seeds(pkg, pkgs_in_seeds))
 
 
     return pkg_support_time
 
+
 if __name__ == "__main__":
     parser = OptionParser()
     parser.add_option("--with-seeds", "", default=False,
-                      action="store_true", 
-                      help="add seed(s) of the package that are responsible for the maintaince time")
+                      action="store_true",
+                      help="add seed(s) of the package that are responsible "
+                           "for the maintaince time")
     parser.add_option("--source-packages", "", default=False,
-                      action="store_true", 
+                      action="store_true",
                       help="show as source pkgs")
     parser.add_option("--hints-file", "", default=None,
                       help="use diffenrt use hints file location")
@@ -275,8 +307,8 @@
     # init
     if len(args) > 0:
         distro = args[0]
-        if distro[0] < 'h':
-            print "ERROR: only hardy or later is supported"
+        if distro in UNSUPPORTED_DISTRO_RELEASED:
+            logging.error("only lucid or later is supported")
             sys.exit(1)
     else:
         distro = "lucid"
@@ -286,26 +318,32 @@
 
     if options.hints_file:
         hints_file = options.hints_file
-        (schema, netloc, path, query, fragment) = urlparse.urlsplit(hints_file)
+        (schema, netloc, path, query, fragment) = urlparse.urlsplit(
+            hints_file)
         if not schema:
             hints_file = "file:%s" % path
     else:
         hints_file = HINTS_DIR_URL % distro
-        
+
     # go over the distros we need to check
     pkg_support_time = {}
     for (name, lts_supported) in DISTRO_NAMES_AND_LTS_SUPPORT:
 
         # get basic structure file
-        structure = get_structure(name, distro)
-    
+        try:
+            structure = get_structure(name, distro)
+        except urllib2.HTTPError:
+            logging.error("Can not get structure for '%s'." % name)
+            continue
+
         # get dicts of pkgname -> support timeframe string
         support_timeframe = SUPPORT_TIMEFRAME
         if lts_supported and distro in LTS_RELEASES:
-            support_timeframe =  SUPPORT_TIMEFRAME_LTS
+            support_timeframe = SUPPORT_TIMEFRAME_LTS
         else:
             support_timeframe = SUPPORT_TIMEFRAME
-        get_packages_support_time(structure, name, pkg_support_time, support_timeframe)
+        get_packages_support_time(
+            structure, name, pkg_support_time, support_timeframe)
 
     # now go over the bits in main that we have not seen (because
     # they are not in any seed and got added manually into "main"
@@ -321,10 +359,11 @@
         for pkg in cache:
             if not pkg.name in pkg_support_time:
                 pkg_support_time[pkg.name] = support_timeframe[-1][0]
-                logging.warn("add package in main but not in seeds %s with %s" % 
-                             (pkg.name, pkg_support_time[pkg.name]))
+                logging.warn(
+                    "add package in main but not in seeds %s with %s" % (
+                        pkg.name, pkg_support_time[pkg.name]))
 
-    # now check the hints file that is used to overwrite 
+    # now check the hints file that is used to overwrite
     # the default seeds
     try:
         for line in urllib2.urlopen(hints_file):
@@ -337,14 +376,15 @@
                     if support_time == 'unsupported':
                         try:
                             del pkg_support_time[pkgname]
-                            sys.stderr.write("hints-file: marking %s unsupported\n" % pkgname)
+                            sys.stderr.write("hints-file: marking %s "
+                                             "unsupported\n" % pkgname)
                         except KeyError:
                             pass
                     else:
                         if pkg_support_time.get(pkgname) != support_time:
                             sys.stderr.write(
                                 "hints-file: changing %s from %s to %s\n" % (
-                                    pkgname,  pkg_support_time.get(pkgname), 
+                                    pkgname, pkg_support_time.get(pkgname),
                                     support_time))
                             pkg_support_time[pkgname] = support_time
             except:
@@ -353,7 +393,7 @@
         if e.code != 404:
             raise
         sys.stderr.write("hints-file: %s gave 404 error\n" % hints_file)
-    
+
     # output suitable for the extra-override file
     for pkgname in sorted(pkg_support_time.keys()):
         # special case, the hints file may contain overrides that
@@ -362,7 +402,7 @@
             print "%s %s %s" % (
                 pkgname, SUPPORT_TAG, pkg_support_time[pkgname])
         else:
-            # go over the supported arches, they are divided in 
+            # go over the supported arches, they are divided in
             # first-class (PRIMARY) and second-class with different
             # support levels
             for arch in SUPPORTED_ARCHES:
@@ -373,10 +413,11 @@
                 if arch in PRIMARY_ARCHES:
                     # arch with full LTS support
                     print "%s %s %s" % (
-                        pkgname_and_arch, SUPPORT_TAG, pkg_support_time[pkgname])
+                        pkgname_and_arch, SUPPORT_TAG,
+                        pkg_support_time[pkgname])
                 else:
                     # not a LTS supported architecture, gets only regular
                     # support_timeframe
                     print "%s %s %s" % (
-                        pkgname_and_arch, SUPPORT_TAG, SUPPORT_TIMEFRAME[0][0])
-                
+                        pkgname_and_arch, SUPPORT_TAG,
+                        SUPPORT_TIMEFRAME[0][0])

=== added directory 'cronscripts/publishing/tests'
=== added directory 'cronscripts/publishing/tests/mock-data'
=== added directory 'cronscripts/publishing/tests/mock-data/mock-bin'
=== added file 'cronscripts/publishing/tests/mock-data/mock-bin/germinate'
--- cronscripts/publishing/tests/mock-data/mock-bin/germinate	1970-01-01 00:00:00 +0000
+++ cronscripts/publishing/tests/mock-data/mock-bin/germinate	2010-10-20 13:23:05 +0000
@@ -0,0 +1,5 @@
+#!/bin/sh
+#
+# This is a mock germinate script that just produces enough (empty)
+# files so that the cron.germinate shell script can run.
+touch structure all all.sources minimal standard

=== added directory 'cronscripts/publishing/tests/mock-data/mock-lp-root'
=== added directory 'cronscripts/publishing/tests/mock-data/mock-lp-root/cronscripts'
=== added directory 'cronscripts/publishing/tests/mock-data/mock-lp-root/cronscripts/publishing'
=== added symlink 'cronscripts/publishing/tests/mock-data/mock-lp-root/cronscripts/publishing/maintenance-check.py'
=== target is '../../../../../maintenance-check.py'
=== added directory 'cronscripts/publishing/tests/mock-data/mock-lp-root/scripts'
=== added directory 'cronscripts/publishing/tests/mock-data/mock-lp-root/scripts/ftpmaster-tools'
=== added file 'cronscripts/publishing/tests/mock-data/mock-lp-root/scripts/ftpmaster-tools/lp-query-distro.py'
--- cronscripts/publishing/tests/mock-data/mock-lp-root/scripts/ftpmaster-tools/lp-query-distro.py	1970-01-01 00:00:00 +0000
+++ cronscripts/publishing/tests/mock-data/mock-lp-root/scripts/ftpmaster-tools/lp-query-distro.py	2010-11-04 13:07:45 +0000
@@ -0,0 +1,27 @@
+#!/usr/bin/python
+#
+# This is a mock version of the lp-query-distro.py script that
+# returns static data so that the cron.germinate shell script
+# can be tested.
+
+import sys
+
+
+def error_and_exit():
+        sys.stderr.write("ERROR: I'm a mock, I only support 'development' "
+                         "and 'supported' as argument\n")
+        sys.exit(1)
+
+
+if __name__ == "__main__":
+        # There is only a very limited subset of arguments that we support,
+        # test for it and error if it looks wrong
+        if len(sys.argv) != 2:
+                error_and_exit()
+        distro = sys.argv[1]
+        if distro == "development":
+                print "natty"
+        elif distro == "supported":
+                print "hardy jaunty karmic lucid maverick"
+        else:
+                error_and_exit()

=== added file 'cronscripts/publishing/tests/test_cron.germinate.py'
--- cronscripts/publishing/tests/test_cron.germinate.py	1970-01-01 00:00:00 +0000
+++ cronscripts/publishing/tests/test_cron.germinate.py	2010-11-04 14:54:26 +0000
@@ -0,0 +1,182 @@
+#!/usr/bin/python
+# Copyright 2010 Canonical Ltd.  This software is licensed under the
+# GNU Affero General Public License version 3 (see the file LICENSE).
+
+"""This is a test for the soyuz cron.germinate script."""
+
+__metaclass__ = type
+
+# mvo: I would love to use this, but it complains about a missing
+#      import for "fixtures" and I can not find a package that
+#      provides this module
+#from lp.testing import TestCase # or TestCaseWithFactory
+from unittest import TestCase
+
+import copy
+import gzip
+import os
+import shutil
+import subprocess
+import tempfile
+import unittest
+
+
+class TestCronGerminate(TestCase):
+
+    DISTS = ["hardy", "lucid", "maverick"]
+    COMPONENTS = ["main", "restricted", "universe", "multiverse"]
+    ARCHES = ["i386", "amd64", "armel", "powerpc"]
+    BASEPATH = os.path.abspath(os.path.dirname(__file__))
+
+    def setUp(self):
+        # Setup a temp archive directory and populate it with the right
+        # sub-directories.
+        self.archive_dir = self.setup_mock_archive_environment()
+        self.ubuntu_misc_dir = os.path.join(self.archive_dir, "ubuntu-misc")
+        self.ubuntu_germinate_dir = os.path.join(
+            self.archive_dir, "ubuntu-germinate")
+        # This is what we pretend to be our current development distro, it
+        # needs to be in sync with the mock lp-query-distro.py.
+        self.populate_mock_archive_environment(
+            self.archive_dir, self.COMPONENTS, self.ARCHES, "natty")
+
+    def tearDown(self):
+        shutil.rmtree(self.archive_dir)
+
+    def create_directory_if_missing(self, directory):
+        """Create the given directory if it does not exist."""
+        if not os.path.exists(directory):
+            os.makedirs(directory)
+
+    def create_directory_list_if_missing(self, directory_list):
+        """Create the given directories from the list if they don't exist."""
+        for directory in directory_list:
+            self.create_directory_if_missing(directory)
+
+    def create_gzip_file(self, filepath, content=""):
+        """Create a gziped file in the given path with the given content.
+           If not content is given a empty file is created.
+        """
+        gz = gzip.GzipFile(filepath, "w")
+        gz.write(content)
+        gz.close()
+
+    def create_file(self, filepath, content=""):
+        """Create a file in the given path with the given content.
+           If not content is given a empty file is created.
+        """
+        f = open(filepath, "w")
+        f.write(content)
+        f.close()
+
+    def setup_mock_archive_environment(self):
+        """Creates a mock archive environment and populate
+           it with the subdirectories that germinate will expect.
+        """
+        tmpdir = tempfile.mkdtemp(prefix="tmp-cron.germinate-test")
+        archive_dir = os.path.join(tmpdir, "mock-data", "ubuntu-archive")
+        ubuntu_misc_dir = os.path.join(archive_dir, "ubuntu-misc")
+        ubuntu_germinate_dir = os.path.join(archive_dir, "ubuntu-germinate")
+        ubuntu_dists_dir = os.path.join(archive_dir, "ubuntu", "dists")
+        self.create_directory_list_if_missing([
+                archive_dir,
+                ubuntu_misc_dir,
+                ubuntu_germinate_dir,
+                ubuntu_dists_dir])
+        return archive_dir
+
+    def populate_mock_archive_environment(self, archive_dir, components_list,
+                                          arches_list, current_devel_distro):
+        """Populates a mock archive environment with empty source packages
+           and empty binary packages.
+        """
+        for component in components_list:
+            # Create the environment for the source packages.
+            targetdir = os.path.join(
+                archive_dir,
+                "ubuntu/dists/%s/%s/source" % (
+                    current_devel_distro, component))
+            self.create_directory_if_missing(targetdir)
+            self.create_gzip_file(os.path.join(targetdir, "Sources.gz"))
+
+            # Create the environment for the binary packages.
+            for arch in arches_list:
+                for subpath in ["", "debian-installer"]:
+                    targetdir = os.path.join(
+                        self.archive_dir,
+                        "ubuntu/dists/%s/%s/%s/binary-%s" % (
+                            current_devel_distro, component, subpath, arch))
+                    self.create_directory_if_missing(targetdir)
+                    self.create_gzip_file(os.path.join(
+                            targetdir, "Packages.gz"))
+
+    def create_fake_environment(self, basepath, archive_dir):
+        """Create a fake process envirionment based on os.environ that
+           sets TEST_ARCHIVEROOT, TEST_LAUNCHPADROOT and modifies PATH
+           to point to the mock lp-bin directory.
+        """
+        fake_environ = copy.copy(os.environ)
+        fake_environ["TEST_ARCHIVEROOT"] = os.path.abspath(
+            os.path.join(archive_dir, "ubuntu"))
+        fake_environ["TEST_LAUNCHPADROOT"] = os.path.abspath(
+            os.path.join(basepath, "mock-data/mock-lp-root"))
+        # Set the PATH in the fake environment so that our mock germinate
+        # is used. We could use the real germinate as well, but that will
+        # slow down the tests a lot and its also not interessting for this
+        # test as we do not use any of the germinate information.
+        fake_environ["PATH"] = "%s:%s" % (
+            os.path.abspath(os.path.join(basepath, "mock-data/mock-bin")),
+            os.environ["PATH"])
+        return fake_environ
+
+    def test_maintenance_update(self):
+        """Test the maintenance-check.py porition of the soyuz cron.germinate
+           shell script by running it inside a fake environment and ensure
+           that it did update the "Support" override information for
+           apt-ftparchive without destroying/modifying the information
+           that the "germinate" script added to it earlier.
+        """
+        # Write into more-extras.overrides to ensure it is alive after we
+        # mucked around.
+        canary = "abrowser Task mock\n"
+        # Build fake environment based on the real one.
+        fake_environ = self.create_fake_environment(
+            self.BASEPATH, self.archive_dir)
+        # Create mock override data files that include the canary string
+        # so that we can test later if it is still there.
+        for dist in self.DISTS:
+            self.create_file(
+                os.path.join(self.ubuntu_misc_dir,
+                             "more-extra.override.%s.main" % dist),
+                canary)
+
+        # Run cron.germinate in the fake environment.
+        cron_germinate_path = os.path.join(
+            self.BASEPATH, "..", "cron.germinate")
+        subprocess.call(
+            [cron_germinate_path], env=fake_environ, cwd=self.BASEPATH)
+
+        # And check the output it generated for correctness.
+        for dist in self.DISTS:
+            supported_override_file = os.path.join(
+                self.ubuntu_misc_dir,
+                "more-extra.override.%s.main.supported" % dist)
+            self.assertTrue(os.path.exists(supported_override_file))
+            main_override_file = os.path.join(
+                self.ubuntu_misc_dir,
+                "more-extra.override.%s.main" % dist)
+            self.assertTrue(canary in open(main_override_file).read(),
+                            msg="canary '%s' is not in file '%s'" % (
+                    canary, main_override_file))
+
+        # Check here if we got the data from maintenance-check.py that
+        # we expected. This is a kernel name from lucid-updates and it
+        # will be valid for 5 years.
+        needle = "linux-image-2.6.32-25-server/i386 Supported 5y"
+        lucid_supported_override_file = os.path.join(
+            self.ubuntu_misc_dir, "more-extra.override.lucid.main")
+        self.assertTrue(needle in open(lucid_supported_override_file).read())
+
+
+if __name__ == "__main__":
+    unittest.main()

